@@ -10,7 +10,7 @@ from ...enums import RequestType, DataInputType
 from ...helper import batch_iterator
 from ...logging import default_logger
 from ...types.document import DocumentSourceType, DocumentContentType, Document
-from ...types.sets.querylang import AcceptQueryLangType
+from ...types.arrays.querylang import AcceptQueryLangType
 
 SingletonDataType = Union[
     DocumentContentType,
@@ -48,13 +48,16 @@ def request_generator(
     :param kwargs: additional arguments
     :yield: request
     """
-    _kwargs = dict(mime_type=mime_type, length=request_size, weight=1.0)
+
+    _kwargs = dict(mime_type=mime_type, weight=1.0, extra_kwargs=kwargs)
 
     try:
         if not isinstance(data, Iterable):
             data = [data]
         for batch in batch_iterator(data, request_size):
-            yield _new_request_from_batch(_kwargs, batch, data_type, mode, queryset)
+            yield _new_request_from_batch(
+                _kwargs, batch, data_type, mode, queryset, **kwargs
+            )
 
     except Exception as ex:
         # must be handled here, as grpc channel wont handle Python exception
