@@ -6,14 +6,15 @@ from typing import Tuple, Dict, List
 
 import numpy as np
 
+from . import FlatRecursiveMixin
 from .encode import BaseEncodeDriver
 from ..types.document.multimodal import MultimodalDocument
 
 if False:
-    from ..types.sets import DocumentSet
+    from ..types.arrays import DocumentArray
 
 
-class MultiModalDriver(BaseEncodeDriver):
+class MultiModalDriver(FlatRecursiveMixin, BaseEncodeDriver):
     """Extract multimodal embeddings from different modalities.
 
     Input-Output ::
@@ -39,8 +40,7 @@ class MultiModalDriver(BaseEncodeDriver):
         - It assumes that every ``chunk`` of a ``document`` belongs to a different modality.
     """
 
-    def __init__(self,
-                 traversal_paths: Tuple[str] = ('r',), *args, **kwargs):
+    def __init__(self, traversal_paths: Tuple[str] = ('r',), *args, **kwargs):
         super().__init__(traversal_paths=traversal_paths, *args, **kwargs)
 
     @property
@@ -49,10 +49,14 @@ class MultiModalDriver(BaseEncodeDriver):
         :return: the list of strings representing the name and order of the modality.
         """
         if not self._exec.positional_modality:
-            raise RuntimeError('Could not know which position of the ndarray to load to each modality')
+            raise RuntimeError(
+                'Could not know which position of the ndarray to load to each modality'
+            )
         return self._exec.positional_modality
 
-    def _get_executor_input_arguments(self, content_by_modality: Dict[str, 'np.ndarray']) -> List['np.ndarray']:
+    def _get_executor_input_arguments(
+        self, content_by_modality: Dict[str, 'np.ndarray']
+    ) -> List['np.ndarray']:
         """From a dictionary ``content_by_modality`` it returns the arguments in the proper order so that they can be
         passed to the executor.
 
@@ -61,18 +65,16 @@ class MultiModalDriver(BaseEncodeDriver):
         """
         return [content_by_modality[modality] for modality in self.positional_modality]
 
-    def _apply_all(
-            self,
-            docs: 'DocumentSet',
-            *args, **kwargs
-    ) -> None:
+    def _apply_all(self, docs: 'DocumentArray', *args, **kwargs) -> None:
         """Apply the driver to each of the Documents in docs.
 
         :param docs: the docs for which a ``multimodal embedding`` will be computed, whose chunks are of different
-        :param *args: args
-        :param **kwargs: kwargs
+        :param args: unused
+        :param kwargs: unused
         """
-        content_by_modality = defaultdict(list)  # array of num_rows equal to num_docs and num_columns equal to
+        content_by_modality = defaultdict(
+            list
+        )  # array of num_rows equal to num_docs and num_columns equal to
 
         valid_docs = []
         for doc in docs:
@@ -83,7 +85,9 @@ class MultiModalDriver(BaseEncodeDriver):
                 for modality in self.positional_modality:
                     content_by_modality[modality].append(doc[modality])
             else:
-                self.logger.warning(f'Invalid doc {doc.id}. Only one chunk per modality is accepted')
+                self.logger.warning(
+                    f'Invalid doc {doc.id}. Only one chunk per modality is accepted'
+                )
 
         if len(valid_docs) > 0:
             # Pass a variable length argument (one argument per array)
@@ -96,6 +100,7 @@ class MultiModalDriver(BaseEncodeDriver):
             if len(valid_docs) != embeds.shape[0]:
                 self.logger.error(
                     f'mismatched {len(valid_docs)} docs from level {valid_docs[0].granularity} '
-                    f'and a {embeds.shape} shape embedding, the first dimension must be the same')
+                    f'and a {embeds.shape} shape embedding, the first dimension must be the same'
+                )
             for doc, embedding in zip(valid_docs, embeds):
                 doc.embedding = embedding
