@@ -1,7 +1,7 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-from typing import Optional, Tuple, Any, Dict, List
+from typing import Optional, Tuple, Dict, List
 
 from . import BaseExecutableDriver, FlatRecursiveMixin, DocsExtractUpdateMixin
 from ..types.document import Document
@@ -22,6 +22,10 @@ class SegmentDriver(DocsExtractUpdateMixin, FlatRecursiveMixin, BaseExecutableDr
             executor, method, traversal_paths=traversal_paths, *args, **kwargs
         )
 
+    @property
+    def _stack_document_content(self):
+        return False
+
     def update_single_doc(self, doc: 'Document', exec_result: List[Dict]) -> None:
         """Update the document's chunks field with executor's returns.
 
