@@ -1,37 +1,96 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-import argparse
+import copy
+import sys
+from abc import abstractmethod
 from argparse import Namespace
 from contextlib import ExitStack
-from typing import Optional, Dict, List, Union, Set
+from itertools import cycle
+from typing import Dict, Union, Set
+from typing import List, Optional
 
 from jina.peapods.zmq import send_ctrl_message
 from jina.types.message.dump import DumpMessage
-
-import copy
-from argparse import Namespace
-from typing import List, Optional
-from itertools import cycle
-
-from ... import __default_host__
-from ...enums import SchedulerType, SocketType, PeaRoleType, PollingType
-from ...helper import get_public_ip, get_internal_ip, random_identity
-from ... import helper
-
 from ..peas import BasePea
-from ...enums import SchedulerType, PodRoleType
+from ... import __default_host__
+from ... import helper
+from ...enums import SchedulerType, PodRoleType, SocketType, PeaRoleType, PollingType
+from ...helper import get_public_ip, get_internal_ip, random_identity
+from ...types.message.common import ControlMessage
 
 
-class BasePod(ExitStack):
+class ExitFIFO(ExitStack):
+    """
+    ExitFIFO changes the exiting order of exitStack to turn it into FIFO.
+
+    .. note::
+    The `__exit__` method is copied literally from `ExitStack` and changed the call:
+    `is_sync, cb = self._exit_callbacks.pop()` to `is_sync, cb = self._exit_callbacks.popleft()`
+
+    """
+
+    def __exit__(self, *exc_details):
+        received_exc = exc_details[0] is not None
+
+        # We manipulate the exception state so it behaves as though
+        # we were actually nesting multiple with statements
+        frame_exc = sys.exc_info()[1]
+
+        def _fix_exception_context(new_exc, old_exc):
+            # Context may not be correct, so find the end of the chain
+            while 1:
+                exc_context = new_exc.__context__
+                if exc_context is old_exc:
+                    # Context is already set correctly (see issue 20317)
+                    return
+                if exc_context is None or exc_context is frame_exc:
+                    break
+                new_exc = exc_context
+            # Change the end of the chain to point to the exception
+            # we expect it to reference
+            new_exc.__context__ = old_exc
+
+        # Callbacks are invoked in LIFO order to match the behaviour of
+        # nested context managers
+        suppressed_exc = False
+        pending_raise = False
+        while self._exit_callbacks:
+            is_sync, cb = self._exit_callbacks.popleft()
+            assert is_sync
+            try:
+                if cb(*exc_details):
+                    suppressed_exc = True
+                    pending_raise = False
+                    exc_details = (None, None, None)
+            except:
+                new_exc_details = sys.exc_info()
+                # simulate the stack of exceptions by setting the context
+                _fix_exception_context(new_exc_details[1], exc_details[1])
+                pending_raise = True
+                exc_details = new_exc_details
+        if pending_raise:
+            try:
+                # bare "raise exc_details[1]" replaces our carefully
+                # set-up context
+                fixed_ctx = exc_details[1].__context__
+                raise exc_details[1]
+            except BaseException:
+                exc_details[1].__context__ = fixed_ctx
+                raise
+        return received_exc and suppressed_exc
+
+
+class BasePod(ExitFIFO):
     """A BasePod is an immutable set of peas. They share the same input and output socket.
     Internally, the peas can run with the process/thread backend.
     They can be also run in their own containers on remote machines.
     """
 
-    def __init__(self, args: Union['argparse.Namespace', Dict], needs: Set[str] = None):
+    def __init__(
+        self, args: Union['Namespace', Dict], needs: Optional[Set[str]] = None
+    ):
         super().__init__()
-        self.peas = []  # type: List['BasePea']
         self.args = args
         self._set_conditional_args(self.args)
         self.needs = (
@@ -53,7 +112,10 @@ class BasePod(ExitStack):
         raise NotImplemented()
 
     def close(self):
-        """Stop all :class:`BasePea` in this BasePod."""
+        """Stop all :class:`BasePea` in this BasePod.
+
+        .. # noqa: DAR201
+        """
         self.__exit__(None, None, None)
 
     @staticmethod
@@ -117,68 +179,6 @@ class BasePod(ExitStack):
         """
         return f'{self.tail_args.host_out}:{self.tail_args.port_out} ({self.tail_args.socket_out!s})'
 
-    def _parse_base_pod_args(
-        self,
-        args,
-        attribute,
-        id_attribute_name,
-        role_type,
-        repetition_attribute,
-        polling_type,
-    ):
-        parsed_args = {'head': None, 'tail': None, attribute: []}
-        if getattr(args, repetition_attribute, 1) > 1:
-            # reasons to separate head and tail from peas is that they
-            # can be deducted based on the previous and next pods
-            self._set_after_to_pass(args)
-            self.is_head_router = True
-            self.is_tail_router = True
-            parsed_args['head'] = BasePod._copy_to_head_args(args, polling_type.is_push)
-            parsed_args['tail'] = BasePod._copy_to_tail_args(args, polling_type.is_push)
-            parsed_args[attribute] = BasePod._set_peas_args(
-                args,
-                role_type=role_type,
-                repetition_attribute=repetition_attribute,
-                id_attribute_name=id_attribute_name,
-                polling_type=polling_type,
-                head_args=parsed_args['head'],
-                tail_args=parsed_args['tail'],
-            )
-        elif (getattr(args, 'uses_before', None) and args.uses_before != '_pass') or (
-            getattr(args, 'uses_after', None) and args.uses_after != '_pass'
-        ):
-            args.scheduling = SchedulerType.ROUND_ROBIN
-            if getattr(args, 'uses_before', None):
-                self.is_head_router = True
-                parsed_args['head'] = self._copy_to_head_args(
-                    args, polling_type.is_push
-                )
-            if getattr(args, 'uses_after', None):
-                self.is_tail_router = True
-                parsed_args['tail'] = self._copy_to_tail_args(
-                    args, polling_type.is_push
-                )
-            parsed_args[attribute] = self._set_peas_args(
-                args,
-                role_type=role_type,
-                repetition_attribute=repetition_attribute,
-                id_attribute_name=id_attribute_name,
-                polling_type=polling_type,
-                head_args=parsed_args.get('head', None),
-                tail_args=parsed_args.get('tail', None),
-            )
-        else:
-            self.is_head_router = False
-            self.is_tail_router = False
-            parsed_args[attribute] = [args]
-
-        # note that peas_args['peas'][0] exist either way and carries the original property
-        return parsed_args
-
-    def _enter_pea(self, pea: 'BasePea') -> None:
-        self.peas.append(pea)
-        self.enter_context(pea)
-
     def __enter__(self) -> 'BasePod':
         return self.start()
 
@@ -186,96 +186,25 @@ class BasePod(ExitStack):
         super().__exit__(exc_type, exc_val, exc_tb)
         self.join()
 
-    @staticmethod
-    def _set_peas_args(
-        args: Namespace,
-        role_type: PeaRoleType,
-        repetition_attribute: str,
-        id_attribute_name: str,
-        polling_type: PollingType,
-        head_args: Optional[Namespace] = None,
-        tail_args: Namespace = None,
-    ) -> List[Namespace]:
-        result = []
-        _host_list = (
-            args.peas_hosts
-            if args.peas_hosts
-            else [
-                args.host,
-            ]
-        )
-
-        for idx, pea_host in zip(
-            range(getattr(args, repetition_attribute)), cycle(_host_list)
-        ):
-            _args = copy.deepcopy(args)
-
-            setattr(_args, id_attribute_name, idx)
-
-            if getattr(args, repetition_attribute) > 1:
-                _args.pea_role = role_type
-                _args.identity = random_identity()
-                if _args.peas_hosts:
-                    _args.host = pea_host
-                if _args.name:
-                    _args.name += f'/{idx}'
-                else:
-                    _args.name = f'{idx}'
-            else:
-                _args.pea_role = PeaRoleType.SINGLETON
-
-            if head_args:
-                _args.port_in = head_args.port_out
-            if tail_args:
-                _args.port_out = tail_args.port_in
-            _args.port_ctrl = helper.random_port()
-            _args.socket_out = SocketType.PUSH_CONNECT
-            if polling_type.is_push:
-                if args.scheduling == SchedulerType.ROUND_ROBIN:
-                    _args.socket_in = SocketType.PULL_CONNECT
-                elif args.scheduling == SchedulerType.LOAD_BALANCE:
-                    _args.socket_in = SocketType.DEALER_CONNECT
-                else:
-                    raise ValueError(
-                        f'{args.scheduling} is not supported as a SchedulerType!'
-                    )
-
-            else:
-                _args.socket_in = SocketType.SUB_CONNECT
-            if head_args:
-                _args.host_in = BasePod._fill_in_host(
-                    bind_args=head_args, connect_args=_args
-                )
-            if tail_args:
-                _args.host_out = BasePod._fill_in_host(
-                    bind_args=tail_args, connect_args=_args
-                )
-
-            result.append(_args)
-        return result
-
-    @staticmethod
-    def _set_after_to_pass(args):
-        raise NotImplemented
-
     @staticmethod
     def _copy_to_head_args(
-        args: Namespace, is_push: bool, as_router: bool = True
+        args: Namespace, polling_type: PollingType, as_router: bool = True
     ) -> Namespace:
         """
         Set the outgoing args of the head router
 
         :param args: basic arguments
-        :param is_push: if true, set socket_out based on the SchedulerType
+        :param polling_type: polling_type can be all or any
         :param as_router: if true, router configuration is applied
         :return: enriched head arguments
         """
 
         _head_args = copy.deepcopy(args)
+        _head_args.polling = polling_type
         _head_args.port_ctrl = helper.random_port()
         _head_args.port_out = helper.random_port()
         _head_args.uses = None
-        if is_push:
+        if polling_type.is_push:
             if args.scheduling == SchedulerType.ROUND_ROBIN:
                 _head_args.socket_out = SocketType.PUSH_BIND
             elif args.scheduling == SchedulerType.LOAD_BALANCE:
@@ -300,17 +229,18 @@ class BasePod(ExitStack):
 
     @staticmethod
     def _copy_to_tail_args(
-        args: Namespace, is_push: bool, as_router: bool = True
+        args: Namespace, polling_type: PollingType, as_router: bool = True
     ) -> Namespace:
         """
         Set the incoming args of the tail router
 
         :param args: configuration for the connection
-        :param is_push: if true, set socket_out based on the SchedulerType
+        :param polling_type: polling type can be any or all
         :param as_router: if true, add router configuration
         :return: enriched arguments
         """
         _tail_args = copy.deepcopy(args)
+        _tail_args.polling_type = polling_type
         _tail_args.port_in = helper.random_port()
         _tail_args.port_ctrl = helper.random_port()
         _tail_args.socket_in = SocketType.PULL_BIND
@@ -323,7 +253,7 @@ class BasePod(ExitStack):
             else:
                 _tail_args.name = f'tail'
             _tail_args.pea_role = PeaRoleType.TAIL
-            _tail_args.num_part = 1 if is_push else args.parallel
+            _tail_args.num_part = 1 if polling_type.is_push else args.parallel
 
         return _tail_args
 
@@ -377,6 +307,27 @@ class BasePod(ExitStack):
             # in this case we (at local) need to know about remote the BIND address
             return bind_args.host
 
+    @abstractmethod
+    def head_args(self):
+        """Get the arguments for the `head` of this BasePod.
+
+        .. # noqa: DAR201
+        """
+        ...
+
+    @abstractmethod
+    def tail_args(self):
+        """Get the arguments for the `tail` of this BasePod.
+
+        .. # noqa: DAR201
+        """
+        ...
+
+    @abstractmethod
+    def join(self):
+        """Wait until all pods and peas exit."""
+        ...
+
 
 class Pod(BasePod):
     """A BasePod is an immutable set of peas, which run in parallel. They share the same input and output socket.
@@ -385,13 +336,17 @@ class Pod(BasePod):
     :param needs: pod names of preceding pods, the output of these pods are going into the input of this pod
     """
 
-    def __init__(self, args: Union['argparse.Namespace', Dict], needs: Set[str] = None):
+    def __init__(
+        self, args: Union['Namespace', Dict], needs: Optional[Set[str]] = None
+    ):
         super().__init__(args, needs)
+        self.peas = []  # type: List['BasePea']
         if isinstance(args, Dict):
             # This is used when a Pod is created in a remote context, where peas & their connections are already given.
             self.peas_args = args
         else:
             self.peas_args = self._parse_args(args)
+        self._activated = False
 
     @property
     def is_singleton(self) -> bool:
@@ -433,18 +388,11 @@ class Pod(BasePod):
     def _parse_args(
         self, args: Namespace
     ) -> Dict[str, Optional[Union[List[Namespace], Namespace]]]:
-        return self._parse_base_pod_args(
-            args,
-            attribute='peas',
-            id_attribute_name='pea_id',
-            role_type=PeaRoleType.PARALLEL,
-            repetition_attribute='parallel',
-            polling_type=getattr(args, 'polling', None),
-        )
+        return self._parse_base_pod_args(args)
 
     @property
     def head_args(self):
-        """Get the arguments for the `head` of this BasePod.
+        """Get the arguments for the `head` of this Pod.
 
 
         .. # noqa: DAR201
@@ -460,7 +408,7 @@ class Pod(BasePod):
 
     @head_args.setter
     def head_args(self, args):
-        """Set the arguments for the `head` of this BasePod.
+        """Set the arguments for the `head` of this Pod.
 
 
         .. # noqa: DAR101
@@ -516,6 +464,19 @@ class Pod(BasePod):
             + self.peas_args['peas']
         )
 
+    @property
+    def _fifo_args(self) -> List[Namespace]:
+        """Get all arguments of all Peas in this BasePod.
+        .. # noqa: DAR201
+        """
+        # For some reason, it seems that using `stack` and having `Head` started after the rest of Peas do not work and
+        # some messages are not received by the inner peas. That's why ExitFIFO is needed
+        return (
+            ([self.peas_args['head']] if self.peas_args['head'] else [])
+            + self.peas_args['peas']
+            + ([self.peas_args['tail']] if self.peas_args['tail'] else [])
+        )
+
     @property
     def num_peas(self) -> int:
         """Get the number of running :class:`BasePea`
@@ -527,6 +488,18 @@ class Pod(BasePod):
     def __eq__(self, other: 'BasePod'):
         return self.num_peas == other.num_peas and self.name == other.name
 
+    def _enter_pea(self, pea: 'BasePea') -> None:
+        self.peas.append(pea)
+        self.enter_context(pea)
+
+    def _activate(self):
+        # order is good. Activate from tail to head
+        for pea in reversed(self.peas):
+            if pea.args.socket_in == SocketType.DEALER_CONNECT:
+                pea.runtime.activate()
+
+        self._activated = True
+
     def start(self) -> 'BasePod':
         """
         Start to run all :class:`BasePea` in this BasePod.
@@ -538,15 +511,17 @@ class Pod(BasePod):
             are properly closed.
         """
         if getattr(self.args, 'noblock_on_start', False):
-            for _args in self.all_args:
+            for _args in self._fifo_args:
                 _args.noblock_on_start = True
                 self._enter_pea(BasePea(_args))
             # now rely on higher level to call `wait_start_success`
             return self
         else:
             try:
-                for _args in self.all_args:
+                for _args in self._fifo_args:
                     self._enter_pea(BasePea(_args))
+
+                self._activate()
             except:
                 self.close()
                 raise
@@ -566,6 +541,7 @@ class Pod(BasePod):
         try:
             for p in self.peas:
                 p.wait_start_success()
+            self._activate()
         except:
             self.close()
             raise
@@ -575,10 +551,12 @@ class Pod(BasePod):
         try:
             for p in self.peas:
                 p.join()
+                self._activated = False
         except KeyboardInterrupt:
             pass
         finally:
             self.peas.clear()
+            self._activated = False
 
     @property
     def is_ready(self) -> bool:
@@ -590,7 +568,7 @@ class Pod(BasePod):
 
         .. # noqa: DAR201
         """
-        return all(p.is_ready.is_set() for p in self.peas)
+        return all(p.is_ready.is_set() for p in self.peas) and self._activated
 
     def _set_after_to_pass(self, args):
         # TODO: check if needed
@@ -599,17 +577,123 @@ class Pod(BasePod):
             # ONLY reset when it is push
             args.uses_after = '_pass'
 
-    def dump(self, path, shards, timeout):
+    @staticmethod
+    def _set_peas_args(
+        args: Namespace,
+        head_args: Optional[Namespace] = None,
+        tail_args: Namespace = None,
+    ) -> List[Namespace]:
+        result = []
+        _host_list = (
+            args.peas_hosts
+            if args.peas_hosts
+            else [
+                args.host,
+            ]
+        )
+
+        for idx, pea_host in zip(range(args.parallel), cycle(_host_list)):
+            _args = copy.deepcopy(args)
+            _args.pea_id = idx
+
+            if args.parallel > 1:
+                _args.pea_role = PeaRoleType.PARALLEL
+                _args.identity = random_identity()
+                if _args.peas_hosts:
+                    _args.host = pea_host
+                if _args.name:
+                    _args.name += f'/{idx}'
+                else:
+                    _args.name = f'{idx}'
+            else:
+                _args.pea_role = PeaRoleType.SINGLETON
+
+            if head_args:
+                _args.port_in = head_args.port_out
+            if tail_args:
+                _args.port_out = tail_args.port_in
+            _args.port_ctrl = helper.random_port()
+            _args.socket_out = SocketType.PUSH_CONNECT
+            if args.polling.is_push:
+                if args.scheduling == SchedulerType.ROUND_ROBIN:
+                    _args.socket_in = SocketType.PULL_CONNECT
+                elif args.scheduling == SchedulerType.LOAD_BALANCE:
+                    _args.socket_in = SocketType.DEALER_CONNECT
+                else:
+                    raise ValueError(
+                        f'{args.scheduling} is not supported as a SchedulerType!'
+                    )
+
+            else:
+                _args.socket_in = SocketType.SUB_CONNECT
+            if head_args:
+                _args.host_in = BasePod._fill_in_host(
+                    bind_args=head_args, connect_args=_args
+                )
+            if tail_args:
+                _args.host_out = BasePod._fill_in_host(
+                    bind_args=tail_args, connect_args=_args
+                )
+
+            result.append(_args)
+        return result
+
+    def _parse_base_pod_args(self, args):
+        parsed_args = {'head': None, 'tail': None, 'peas': []}
+        if getattr(args, 'parallel', 1) > 1:
+            # reasons to separate head and tail from peas is that they
+            # can be deducted based on the previous and next pods
+            self.is_head_router = True
+            self.is_tail_router = True
+            parsed_args['head'] = BasePod._copy_to_head_args(args, args.polling)
+            parsed_args['tail'] = BasePod._copy_to_tail_args(args, args.polling)
+            parsed_args['peas'] = self._set_peas_args(
+                args,
+                head_args=parsed_args['head'],
+                tail_args=parsed_args['tail'],
+            )
+        elif (getattr(args, 'uses_before', None) and args.uses_before != '_pass') or (
+            getattr(args, 'uses_after', None) and args.uses_after != '_pass'
+        ):
+            args.scheduling = SchedulerType.ROUND_ROBIN
+            if getattr(args, 'uses_before', None):
+                self.is_head_router = True
+                parsed_args['head'] = self._copy_to_head_args(args, args.polling)
+            if getattr(args, 'uses_after', None):
+                self.is_tail_router = True
+                parsed_args['tail'] = self._copy_to_tail_args(args, args.polling)
+            parsed_args['peas'] = self._set_peas_args(
+                args,
+                head_args=parsed_args.get('head', None),
+                tail_args=parsed_args.get('tail', None),
+            )
+        else:
+            self.is_head_router = False
+            self.is_tail_router = False
+            parsed_args['peas'] = [args]
+
+        # note that peas_args['peas'][0] exist either way and carries the original property
+        return parsed_args
+
+    def dump(self, pod_name, dump_path, shards, timeout):
         """Emit a Dump request to its Peas
 
+        :param pod_name: the pod to target
         :param shards: the nr of shards in the dump
-        :param path: the path to which to dump
+        :param dump_path: the path to which to dump
         :param timeout: time to wait (seconds)
         """
         for pea in self.peas:
-            if 'head' not in pea.name and 'tail' not in pea.name:
+            if pea.inner:
                 send_ctrl_message(
                     pea.runtime.ctrl_addr,
-                    DumpMessage(path=path, shards=shards),
+                    ControlMessage(
+                        command='DUMP',
+                        pod_name=pod_name,
+                        args={
+                            'dump_path': dump_path,
+                            'shards': shards,
+                        },
+                    ),
                     timeout=timeout,
                 )
