@@ -13,7 +13,7 @@ class ControlMessage(Message):
     """
     Class of the protobuf message.
 
-    :param command: Command with string content. (e.g. 'IDLE', 'TERMINATE', 'STATUS')
+    :param command: Command with string content. (e.g. 'IDLE', 'CANCEL', 'TERMINATE', 'STATUS')
     :param pod_name: Name of the current pod.
     :param identity: The identity of the current pod
     :param args: Additional positional arguments which are just used for the parent initialization
@@ -36,3 +36,8 @@ class ControlMessage(Message):
             None, req, pod_name=pod_name, identity=identity, *args, **kwargs
         )
         req.request_type = 'control'
+        args = kwargs.get('args', None)
+        if args:
+            req.args = args
+
+        req.targets.extend([pod_name])
